@@ -5,10 +5,9 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.12';
+our $VERSION = '0.20';
 
 use XSLoader;
-use Scope::Guard;
 
 use base qw(Exporter);
 
@@ -17,54 +16,43 @@ our @EXPORT_TAGS = (':all' => [ @EXPORT_OK ]);
 
 XSLoader::load(__PACKAGE__, $VERSION);
 
+sub my_hints() {
+    # set HINT_LOCALIZE_HH (0x20000)
+    $^H |= 0x20000;
+    return \%^H;
+}
+
 sub new_scope(;$) {
     my $caller = shift || caller;
-
-    $^H |= 0x80020000;
-    $^H{'Devel::Pragma'} = 1;
+    my $hints = my_hints;
 
     # this is %^H as an integer - it changes as scopes are entered/exited i.e. it's a unique
-    # identifier for the currently compiling scope (the scope in which new_scope 
+    # identifier for the currently-compiling scope (the scope in which new_scope 
     # is called)
     #
     # we don't need to stack/unstack it in %^H as %^H itself takes care of that
-    # note: we need to call this *after* %^H is referenced (and possibly autovivified) by my_hints
+    # note: we need to call this *after* %^H is referenced (and possibly autovivified) above
     #
     # every time new_scope is called, we write this scope ID to $^H{"Devel::Pragma::Scope::$caller"}.
     # if $^H{"Devel::Pragma::Scope::$caller"} == _scope() (i.e. the stored scope ID is the same as the
     # current scope ID), then we're augmenting the current scope; otherwise we're in a new scope - i.e.
     # a nested or outer scope that didn't previously "use MyPragma"
 
-    # set HINT_LOCALIZE_HH (0x20000) + an unused bit (0x80000000) so that
-    # this module (which can't use itself) can work around the %^H bug
-
     my $current_scope = _scope();
     my $id = "Devel::Pragma::Scope::$caller";
-    my $old_scope = exists($^H{$id}) ? $^H{$id} : 0;
+    my $old_scope = exists($hints->{$id}) ? $hints->{$id} : 0;
     my $new_scope; # is this a scope in which new_scope has not previously been called?
 
     if ($current_scope == $old_scope) {
         $new_scope = 0;
     } else {
-        $^H{$id} = $current_scope;
+        $hints->{$id} = $current_scope;
         $new_scope = 1;
     }
 
     return $new_scope;
 }
 
-sub my_hints() {
-    if (new_scope) {
-        my $guard = Scope::Guard->new(\&_leave);
-
-        $^H{$guard} = $guard;
-
-        _enter();
-    }
-
-    return \%^H;
-}
-
 1;
 
 __END__
@@ -97,6 +85,12 @@ This module provides helper functions for developers of lexical pragmas. These c
 perl (from 5.6.0), which have limited support for lexical pragmas, and in the most recent versions, which have improved
 support.
 
+In addition to the helper functions, this module applies a global fix that makes %^H lexically-scoped
+rather than dynamically-scoped. Until perl change #33311, which isn't currently available in any stable
+perl release, values set in %^H are visible in modules loaded by C<use>, C<require> and C<do FILE>.
+This makes pragmas leak from the scope in which they're meant to be enabled into scopes in which
+they're not. This module applies a fix which ensures that values in %^H no longer leak across file boundaries.
+
 =head1 EXPORTS
 
 Devel::Pragma exports the following functions on demand. They can all be imported at once by using the C<:all> tag. e.g.
@@ -105,18 +99,10 @@ Devel::Pragma exports the following functions on demand. They can all be importe
 
 =head2 my_hints
 
-Until perl change #33311, which isn't currently available in any stable perl release, %^H is dynamically-scoped,
-rather than lexically-scoped. This means that values set in %^H are visible in modules loaded by C<use>.
-This makes pragmas leak from the scope in which they're meant to be enabled into scopes in which they're
-not. C<my_hints> fixes that by making %^H lexically scoped i.e. it prevents %^H leaking across file boundaries.
-
-C<my_hints> installs versions of perl's C<require> and C<do EXPR> builtins in the
-currently-compiling scope which clear %^H before they execute and restore the previous %^H afterwards.
-Thus it can be thought of a lexically-scoped backport of change #33311.
-
-Note that C<my_hints> also sets the $^H bit that "localizes" (or in this case "lexicalizes") %^H.
-
-The return value is a reference to %^H.
+C<my_hints> sets the appropriate flag to make %^H lexically-scoped, and returns a reference to %^H. More
+precisely, it sets the flag in $^H that makes %^H copy and restore its values as scopes are entered and exited.
+The fix that clears %^H before C<use>, C<require> and C<do FILE> statements is applied globally when
+the Devel::Pragma module is first loaded.
 
 =head2 new_scope
 
@@ -141,8 +127,8 @@ If not supplied, the identifier defaults to the name of the calling package.
 
 =head2 ccstash
 
-This returns the name of the currently compiling stash. It can be used as a replacement for the scalar form of
-C<caller> to provide the name of the package in which C<use MyPragma> is called, but unlike C<caller> it
+This returns the name of the currently-compiling stash. It can be used as a replacement for the scalar form of
+C<caller> to provide the name of the package in which C<use MyPragma> is called. Unlike C<caller> it
 returns the same value regardless of the number of intervening calls before C<MyPragma::import>
 is reached.
 
@@ -183,16 +169,20 @@ the call to C<MySuperPragma::import> (via C<MySubPragma::import>) takes place i.
 
 =head1 VERSION
 
-0.12
+0.20
 
 =head1 SEE ALSO
 
 =over
 
+=item * L<pragma|pragma>
+
 =item * L<perlpragma|perlpragma>
 
 =item * L<perlvar|perlvar>
 
+=item * L<B::Hooks::EndOfScope|B::Hooks::EndOfScope>
+
 =item * L<Devel::Hints|Devel::Hints>
 
 =item * http://tinyurl.com/45pwzo
@@ -205,7 +195,7 @@ chocolateboy <chocolate.boy@email.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2008 by chocolateboy
+Copyright (C) 2008-2009 by chocolateboy
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
